// PG_EntryListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_EntryListCtrl.h"
#include "QEntry.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl

PG_EntryListCtrl::PG_EntryListCtrl()
{
	m_pEntryReader = NULL;
}

PG_EntryListCtrl::~PG_EntryListCtrl()
{
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::Create
void PG_EntryListCtrl::Create( CWnd *pParentWnd, RECT &rect, UINT nId )
{
	extern CPCPages32App theApp;

  CListCtrl::Create( WS_VISIBLE|LVS_REPORT|WS_BORDER, rect, pParentWnd, nId );
	SetExtendedStyle( GetExtendedStyle() | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP );

 	//
	// Load images
	m_imageList.Create( IDB_ENTRY_ICONS, 19, 0, RGB( 0xff, 0xff, 0xff ) );
	SetImageList( &m_imageList, LVSIL_SMALL );

	//
	// Load configuration from registry
	if ( m_elConfig.ReadConfiguration() == FALSE )
		theApp.ErrorBox( PGS_REGINIT_FAILED );

	//
	// Create the columns according to loaded configuration information
	QStringList colList;
	m_elConfig.GetColumnList( colList );

  LPCTSTR str;
  UINT i = 0;

  POSITION pos = colList.GetHeadPosition();
  while ( pos != NULL )
    {
      str = ( LPCTSTR ) colList.GetNext( pos );
      InsertColumn( i, str, LVCFMT_LEFT, 200, i );
      i++;
    }
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::AddEntry
// Add an entry to the list. This takes control of the supplied entry.
void PG_EntryListCtrl::AddEntry( QEntry *pEntry )
{
	LV_ITEM lvi;

  //
  // Set base entry attributes (name and entry data pointer)
  lvi.iItem = 0;
  lvi.lParam = reinterpret_cast<long>( pEntry );

	//
	// Get string entry for first column
	CString strVal;
	m_elConfig.GetColumnValue( pEntry, 0, strVal );
	lvi.pszText = strdup( strVal );

  //
  // Add the base entry (i.e. column 0) 
  lvi.mask = LVIF_TEXT|LVIF_PARAM;
  lvi.iSubItem = 0;
  int nItemID = InsertItem( &lvi );
	free( lvi.pszText );

	//
	// Add remaining columns
	QStringList colList;
	UINT nColCount = m_elConfig.GetColumnList( colList );
	for ( UINT nColID = 1; nColID < nColCount; nColID++ )
		{
			m_elConfig.GetColumnValue( pEntry, nColID, strVal );
			lvi.pszText = strdup( strVal );

			lvi.mask = LVIF_TEXT;
			lvi.iSubItem = nColID;

			SetItem( &lvi );
			free( lvi.pszText );
		}

	//
	// Add entry to list
	m_entryList.AddHead( pEntry );
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::AddEntryList
// Add a number of entries to the list. This takes control of the
// supplied entries.
void PG_EntryListCtrl::AddEntryList( PtrList<QEntry> &rEntryList )
{
	POSITION pos = rEntryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = rEntryList.GetNext( pos );
			AddEntry( pEntry );
		}
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::RemoveEntry
void PG_EntryListCtrl::RemoveEntry( QDN &dn )
{
	// XXX PG_EntryListCtrl::RemoveEntry unimplemented
}

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl::RemoveAllEntries
void PG_EntryListCtrl::RemoveAllEntries()
{
	if ( m_entryList.IsEmpty() == TRUE )
		return;

	DeleteAllItems();

	POSITION pos = m_entryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = m_entryList.GetNext( pos );
			delete pEntry;
		}

	m_entryList.RemoveAll();
}

BEGIN_MESSAGE_MAP(PG_EntryListCtrl, CListCtrl)
	//{{AFX_MSG_MAP(PG_EntryListCtrl)
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT(HDN_ITEMDBLCLICK, OnItemdblclick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_NOTIFY_REFLECT(LVN_BEGINDRAG, OnBegindrag)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PG_EntryListCtrl message handlers

void PG_EntryListCtrl::OnDestroy() 
{
	RemoveAllEntries();
	CListCtrl::OnDestroy();
}

void PG_EntryListCtrl::OnItemdblclick( NMHDR* pNMHDR, LRESULT* pResult ) 
{
}

void PG_EntryListCtrl::OnDblclk( NMHDR* pNMHDR, LRESULT* pResult ) 
{
	if ( m_pEntryReader != NULL &&
				m_pEntryReader->Invoked() == TRUE &&
				m_pEntryReader->Finished() == FALSE )
		{
			//
			// Already reading an entry!
			*pResult = 0;
			return;
		}

	if ( m_pEntryReader != NULL )
		{
			delete m_pEntryReader;
			m_pEntryReader = NULL;
		}

	//
	// Get the selected list item
	int item = GetNextItem( -1, LVNI_SELECTED );
  LV_ITEM lvi;

  if (item < 0)
    {
      *pResult = 0;
      return;
    }

  //
  // Get entry pointer of the item
  lvi.iItem = item;
  lvi.mask = LVIF_PARAM;

  GetItem( &lvi );
  QEntry *pEntry = reinterpret_cast<QEntry *>( lvi.lParam );

  //
  // Request a read and display operation
	extern CPCPages32App theApp;
	theApp.SetMessageText( "" );

	m_pEntryReader = 
		new QReadEntry
			( theApp.DS(), theApp.DS()->GenRequestID(), &m_readHandler, pEntry->DN() );

	QError *pErr = NULL;
	if ( m_pEntryReader->Invoke( pErr ) == FALSE )
		{
#ifdef _DEBUG
			CString errStr;
			pErr->Print( errStr );

			TRACE
				( " *** PG_EntryListCtrl::OnDblclk()\
					\nFailed to read an entry from the directory, error message is:\
					\n%s\n\n",
					errStr );
#endif
			theApp.ErrorBox( PGS_DSREAD_FAILED );
		}

	if ( pErr != NULL )
		delete pErr;

	*pResult = 0;
}

void PG_EntryListCtrl::OnBegindrag( NMHDR* pNMHDR, LRESULT* pResult ) 
{
}
