// PG_FormHandler.cpp: implementation of the PG_FormHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCPages32.h"
#include "QFormSearch.h"
#include "PG_FormHandler.h"
#include "QFormCompletionReport.h"
#include "MainFrm.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PG_FormHandler::PG_FormHandler()
{
	m_nMatchNum = 0;
}

PG_FormHandler::~PG_FormHandler()
{
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::HandleActivity
// Handle activity during the course of a form search
void PG_FormHandler::HandleActivity( QDUAActivity event, QRequestMsg *pMsg )
{
	switch ( event )
		{
		case DUAreport:
			Report( pMsg );
			break;

		case DUAresult:
			Result( pMsg );
			break;

		case DUAerror:
			Error( pMsg );
			break;

		case DUAcomplete:
			Complete( pMsg );
			break;
		}
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::Report
//
// Some progress has been made. Process the error and report to
// the user.
void PG_FormHandler::Report( QRequestMsg *pMsg )
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	QFormReport *pReport = ( QFormReport * ) pMsg->Message();

	CString str;
	PtrList<QEntry> entryList;

	switch ( pReport->Type() )
		{
		case QRInterimMatches:
			pReport->GetEntries( entryList );
			ReportInterimMatches( entryList );
			break;

		case QRReferrals:
			break;

		case QRError:
			break;

		case QROpCount:
			ReportOpCount( pReport );
			break;
		}

	delete pReport;
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::ReportOpCount
//
// Some interim matches have been made. These are displayed in
// the browser tree.
void PG_FormHandler::ReportOpCount( QFormReport *pReport )
{
	extern CPCPages32App theApp;
	int nInvoked, nFailed;

	pReport->GetOpCount( nInvoked, nFailed );
	theApp.MainFrame()->SetOpCounts( nInvoked, nFailed );
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::ReportInterimMatches
//
// Some interim matches have been made. These are displayed in
// the browser tree.
void PG_FormHandler::ReportInterimMatches( PtrList<QEntry> &rEntryList )
{
	extern CPCPages32App theApp;

	POSITION pos = rEntryList.GetHeadPosition();
	while ( pos != NULL )
		{
			QEntry *pEntry = rEntryList.GetNext( pos );
			theApp.DITDisplay().AddEntry( pEntry, TRUE );
			delete pEntry;
		}
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::Error
//
// The search has terminated in error.
void PG_FormHandler::Error( QRequestMsg *pMsg )
{
	// XXX PG_FormHandler::Error unimplemented
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::Complete
//
// The search has completed successfully.
void PG_FormHandler::Complete( QRequestMsg *pMsg )
{
	extern CPCPages32App theApp;
	QFormCompletionReport *pReport = 
		static_cast<QFormCompletionReport *>( pMsg->Message() ) ;

	CString tmpStr, msgStr;
	tmpStr.LoadString( PGS_STATREP_SRCH_COMPLETE );

	msgStr.Format( tmpStr, m_nMatchNum, pReport->OpsInvoked(), pReport->OpsFailed() );
	theApp.SetMessageText( msgStr );

	theApp.FlashMessageText();
	delete pReport;
}

//////////////////////////////////////////////////////////////////////
// PG_FormHandler::Result
//
// The search has returned one or more matches.
void PG_FormHandler::Result( QRequestMsg *pMsg )
{
	extern CPCPages32App theApp;
	QFormResult *pResult = ( QFormResult * ) pMsg->Message();

	//
	// Get the list of entries
	PtrList<QEntry> entryList;
	pResult->GetEntries( entryList );

	//
	// Count them and report
	m_nMatchNum += entryList.GetCount();
	
	CString tmpStr, msgStr;
	tmpStr.LoadString( PGS_STATREP_MATCHES_MADE );

	msgStr.Format( tmpStr, m_nMatchNum );
	theApp.SetMessageText( msgStr );

	//
	// Add them to the displayed search results list
	theApp.EntryListDisplay().AddEntryList( entryList );
	delete pResult;
}

void PG_FormHandler::Reset()
{
	m_nMatchNum = 0;
}
