// PG_FormWin.cpp : implementation file
//

#include "stdafx.h"
#include "PCPages32.h"
#include "PG_FormWin.h"
#include "qutil.h"
#include "QFormSearch.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define FORM_CONTROL_SPACING 8
#define FORM_PADDING 8
#define STATIC_TITLE_ID	101
#define STATIC_HELP_ID	102
#define SUBMIT_ID				103
#define CLEAR_ID				104
#define FIRST_EDIT_CONTROL 105

/////////////////////////////////////////////////////////////////////////////
// PG_FormField aggregates a label and edit control

PG_FormField::PG_FormField
	( CString &name ,
		PG_FormWin *pFormWin,
		QFormInput *pInput,
		UINT label_id,
		UINT edit_id,
		UINT left,
		UINT right,
		UINT top )
{
	field_name = name;
	m_pInput = pInput;
	m_pFormWin = pFormWin;

	pFormWin->GetClientRect( bound_rect );
	bound_rect.top = top + FORM_CONTROL_SPACING;
	bound_rect.left = FORM_CONTROL_SPACING;
	bound_rect.right -= FORM_CONTROL_SPACING;

	m_nLabelID = label_id;
	m_nEditID = edit_id;

	//
	// Create field label
	CDC *pDC = pFormWin->GetDC();
	CSize sz = pDC->GetTextExtent( name );
	pFormWin->ReleaseDC( pDC );
	
	RECT rect;
	rect.left = bound_rect.left;
	rect.right = rect.left + sz.cx;
	rect.top = bound_rect.top;
	rect.bottom = bound_rect.bottom = bound_rect.top + sz.cy;

	label.Create( name, WS_CHILD|WS_VISIBLE, rect, pFormWin, label_id );

	//
	// Create the edit control
	rect.left = rect.right + FORM_CONTROL_SPACING;
	rect.right = bound_rect.right;

	int border_width = GetSystemMetrics(SM_CYEDGE);
	rect.top -= ( border_width + 2 );
	rect.bottom += ( border_width + 2 );

	if ( rect.right <= rect.left )
		rect.right = rect.left + FORM_CONTROL_SPACING;

	DWORD dwStyle = WS_CHILD|WS_VISIBLE|WS_TABSTOP;
	if ( label_id == FIRST_EDIT_CONTROL )
		dwStyle |= WS_GROUP;

	edit.Create( dwStyle, rect, pFormWin, edit_id );
	edit.ModifyStyleEx( 0, WS_EX_CLIENTEDGE );

	label.SetFont( pFormWin->m_pNormalFont );
	edit.SetFont( pFormWin->m_pNormalFont );

	//
	// Get the extents of the label and edit controls
	SizeControls( pFormWin, left, right, top );
}

PG_FormField::~PG_FormField()
{
}

/////////////////////////////////////////////////////////////////////////////
// Position the control at the given co-ordinates and compute 
// and set the internal bounding rectangle.
void PG_FormField::SizeControls( CWnd *parent, UINT left, UINT right, UINT top )
{
	CDC *dc = parent->GetDC();
	CSize label_size = dc->GetTextExtent( field_name, field_name.GetLength() );
	CSize edit_size = label_size;

	edit_size.cy += (GetSystemMetrics(SM_CYEDGE) * 2);

	CRect label_rect
		( left, top + GetSystemMetrics(SM_CYEDGE), left + label_size.cx, top + label_size.cy );

	CRect edit_rect
		( left + label_size.cx, top, right, top + edit_size.cy );

	label.MoveWindow
		( label_rect.left, label_rect.top, 
			label_rect.right - label_rect.left,	
			label_rect.bottom - label_rect.top );

	edit.MoveWindow
		( edit_rect.left, edit_rect.top, 
			edit_rect.right - edit_rect.left,	
			edit_rect.bottom - edit_rect.top );

	bound_rect.top = top;
	bound_rect.left = left;
	bound_rect.bottom = top + edit_size.cy;
	bound_rect.right = right;
}

/////////////////////////////////////////////////////////////////////////////
// Return the input field name
CString PG_FormField::FieldName()
{
	return field_name;
}

/////////////////////////////////////////////////////////////////////////////
// Clear the edit field
void PG_FormField::Clear()
{
	edit.SetWindowText("");
}

/////////////////////////////////////////////////////////////////////////////
// Set the form field input
void PG_FormField::SetFieldValue()
{
	CString str;
	m_pFormWin->GetDlgItemText( m_nEditID, str );
	m_pInput->SetFieldValue( field_name, str );
}

/////////////////////////////////////////////////////////////////////////////
// Return the string value entered into this input field
void PG_FormField::GetValue( CString &str )
{
	edit.GetWindowText( str );
}

/////////////////////////////////////////////////////////////////////////////
// Get the field's display rectangle
void PG_FormField::GetRect( CRect &rect )
{
	rect = bound_rect;
}

/////////////////////////////////////////////////////////////////////////////
// PG_FormWin

PG_FormWin::PG_FormWin( QForm *pForm )
{
	m_pForm = pForm;
	m_pRequest = NULL;
	m_pForm->BuildInputFields( &target_ffi, context_ffi_list );
	m_pNormalFont = NULL;
	m_pBoldFont = NULL;
	id_acc = FIRST_EDIT_CONTROL;
}

PG_FormWin::~PG_FormWin()
{
	if ( m_pRequest != NULL )
		{
			m_pRequest = NULL;
		}
}

/////////////////////////////////////////////////////////////////////////////
// Define a better looking font for this window
void PG_FormWin::CreateFonts()
{
	//
	// Create normal font
  CFont *font = new CFont;
  CDC *dc = GetDC();
  int fnth = 12; //MulDiv(9, dc->GetDeviceCaps(LOGPIXELSY), 72);

  BOOL font_ok = font->CreateFont
    ( -fnth, 0,  // Width and height
			 0, 0,      // Escapement and orientation
			 0, 0, 0, 0, // Weight, italic, underline, strikeout
			 ANSI_CHARSET,
			 OUT_DEFAULT_PRECIS,
			 CLIP_DEFAULT_PRECIS,
			 DEFAULT_QUALITY,
			 VARIABLE_PITCH|FF_DONTCARE,
			 "Arial");

#ifdef _DEBUG
  if ( font_ok == FALSE )
		TRACE0( "Failed to set new font for a PG_FormWin.\n" );
#endif 
  
  m_pNormalFont = font;

	//
	// Create bold font
  font = new CFont;
  fnth = 12; //MulDiv(9, dc->GetDeviceCaps(LOGPIXELSY), 72);

  font_ok = font->CreateFont
    ( -fnth, 0,  // Width and height
			 0, 0,      // Escapement and orientation
			 FW_BOLD, 0, 0, 0, // Weight, italic, underline, strikeout
			 ANSI_CHARSET,
			 OUT_DEFAULT_PRECIS,
			 CLIP_DEFAULT_PRECIS,
			 DEFAULT_QUALITY,
			 VARIABLE_PITCH|FF_DONTCARE,
			 "Arial");

#ifdef _DEBUG
  if ( font_ok == FALSE )
		TRACE0( "Failed to set new font for a PG_FormWin.\n" );
#endif 
  
  m_pBoldFont = font;

	//
	// Set the font and then we're all done
  SetFont( m_pNormalFont );
  ReleaseDC( dc );
}

/////////////////////////////////////////////////////////////////////////////
// Create the form window and contained input fields.
BOOL PG_FormWin::Create( CWnd *parent, CRect &rect, UINT id )
{
	//
	// Create the base window
	CWnd::Create(	NULL, m_pForm->Name(), WS_CHILD, rect, parent, id );
	CreateFonts();

	//
	// Create the child form controls, beginning with the target input fields
	UINT left = FORM_PADDING;
	UINT right = FORM_PADDING;
	UINT top = FORM_PADDING;

	//
	// Create the static help strings
	CClientDC dc( this );
	CSize sz;
	CRect helpRct;
			
	if ( !m_pForm->TipString().IsEmpty() && m_pForm->TipString() != "" )
		{
			//
			// Calculate the extents of the title string
			sz = dc.GetTextExtent( m_pForm->TipString() );
			helpRct.SetRect( left, top, right, top + sz.cy );

			m_staticTitle.Create
				( m_pForm->TipString(), WS_CHILD|WS_VISIBLE, helpRct, this, STATIC_HELP_ID );
			
			m_staticTitle.SetFont( m_pBoldFont );
			top += ( sz.cy + FORM_PADDING );
		}

	if ( !m_pForm->HelpString().IsEmpty() && m_pForm->HelpString() != "" )
		{
			//
			// ...and the help string
			sz = dc.GetTextExtent( m_pForm->HelpString() );

			helpRct.SetRect( left, top, right, top + sz.cy );
			m_staticHelp.Create
				( m_pForm->HelpString(), WS_CHILD|WS_VISIBLE, helpRct, this, STATIC_HELP_ID );

			if ( m_pNormalFont != NULL )
				m_staticHelp.SetFont( m_pNormalFont );

			top += ( sz.cy + FORM_PADDING );
		}

	//
	// Target input fields
	QStringList field_name_list;
	target_ffi->GetFieldNames( field_name_list );

	if ( CreateFields( field_name_list, left, right, top, target_ffi ) == FALSE )
		return FALSE;

	//
	// Context input fields
	POSITION pos = context_ffi_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormInput *context_ffi = context_ffi_list.GetNext( pos );
			context_ffi->GetFieldNames( field_name_list );

			if ( CreateFields( field_name_list, left, right, top, context_ffi ) == FALSE )
				return FALSE;
		}

	//
	// Create "Submit" and "Clear" buttons
	CDC *pDC = GetDC();
	CString labelStr;
	
	PG_FormField *pField = field_list.GetTail();
	CRect btnRect;
	pField->GetRect( btnRect );

	//
	// First the "Submit" button
	btnRect.left = FORM_CONTROL_SPACING;
	btnRect.top += FORM_CONTROL_SPACING;

	labelStr.LoadString( PGS_SUBMIT_BUTTON );
  CSize size = pDC->GetTextExtent( labelStr, strlen( labelStr ) );

  btnRect.right = btnRect.left + size.cx + ( GetSystemMetrics( SM_CXEDGE ) * 4 );
  btnRect.bottom = btnRect.top + size.cy + ( GetSystemMetrics( SM_CYEDGE ) * 4 );
  
	m_submitButton.Create( labelStr, WS_CHILD|WS_VISIBLE|WS_TABSTOP, btnRect, this, SUBMIT_ID );
  m_submitButton.SetFont( m_pNormalFont );

	//
	// Then the "Clear" button
	labelStr.LoadString( PGS_CLEAR_BUTTON );
  size = pDC->GetTextExtent( labelStr, strlen( labelStr ) );

	btnRect.left = btnRect.right + FORM_CONTROL_SPACING;
  btnRect.right = btnRect.left + size.cx + ( GetSystemMetrics( SM_CXEDGE ) * 4 );
  
	m_clearButton.Create( labelStr, WS_CHILD|WS_VISIBLE|WS_TABSTOP, btnRect, this, CLEAR_ID );
  m_clearButton.SetFont( m_pNormalFont );

	ReleaseDC( pDC );
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Create the form window and contained input fields.
BOOL PG_FormWin::CreateFields
	( QStringList &field_name_list, UINT &left, UINT &right, UINT &top, QFormInput *ffi )
{	
	POSITION pos = field_name_list.GetHeadPosition();
	while ( pos != NULL )
		{
			CString field_name = field_name_list.GetNext( pos );
			PG_FormField *field = 
				new PG_FormField
					( field_name,	this, ffi, id_acc,	id_acc+1, left, right, top );

			id_acc += 2;
			field_list.AddTail( field );

			CRect rect;
			field->GetRect( rect );
			top = rect.bottom + FORM_PADDING;
		}

	return TRUE;
}

BEGIN_MESSAGE_MAP(PG_FormWin, CWnd)
	//{{AFX_MSG_MAP(PG_FormWin)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
  ON_BN_CLICKED(SUBMIT_ID, OnSubmit)
  ON_BN_CLICKED(CLEAR_ID, OnClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// PG_FormWin message handlers

/////////////////////////////////////////////////////////////////////////////
// Clean up window related stuff
void PG_FormWin::OnDestroy() 
{
	CWnd::OnDestroy();

  if ( m_pNormalFont != NULL )
		{
			for ( int i = FIRST_EDIT_CONTROL; i < id_acc; i++ )
				{
					CWnd *wnd = GetDlgItem( i );
					if ( wnd != NULL )
						wnd->SetFont( NULL );
				}

			SetFont( NULL );
			delete m_pNormalFont;
		}

  if ( m_pBoldFont != NULL )
		{
			delete m_pBoldFont;
		}


	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_FormField *field = field_list.GetNext( pos );
			delete field;
		}

	//
	// Delete field inputs
	delete target_ffi;
	pos = context_ffi_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QFormInput *pFFI = context_ffi_list.GetNext( pos );
			delete pFFI;
		}
}

/////////////////////////////////////////////////////////////////////////////
// Resize form input controls
void PG_FormWin::OnSize( UINT nType, int cx, int cy ) 
{
	//
	// Don't do any resizing if child windows have
	// yet to be created.
	if ( m_submitButton.GetSafeHwnd() == NULL )
		{
			CWnd::OnSize( nType, cx, cy );
			return;
		}

	//
	// All OK. Get starting co-ordinates and bounding rect.
	CRect rect;
	GetClientRect( &rect );

	UINT left = rect.left + FORM_PADDING;
	UINT right = rect.right - FORM_PADDING;
	UINT top = rect.top + FORM_PADDING;
	
	//
	// Calculate the extents of the help string
	if ( !m_pForm->TipString().IsEmpty() && m_pForm->TipString() != "" )
		{
			CClientDC dc( &m_staticTitle );
			CRect helpRect( left, top, right, top + 16 );
			dc.DrawText( m_pForm->TipString(), &helpRect,	DT_CALCRECT|DT_WORDBREAK );

			m_staticTitle.MoveWindow( &helpRect );
			top = helpRect.bottom + 8;
		}

	if ( !m_pForm->HelpString().IsEmpty() && m_pForm->HelpString() != "" )
		{
			CClientDC dc( &m_staticHelp );
			CRect helpRect( left, top, right, top + 16 );
			dc.DrawText( m_pForm->HelpString(), &helpRect,	DT_CALCRECT|DT_WORDBREAK );

			m_staticHelp.MoveWindow( &helpRect );
			top = helpRect.bottom + 8;
		}

	//
	// Resize and reposition fields
	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_FormField *field = field_list.GetNext( pos );
			field->SizeControls( this, left, right, top );

			field->GetRect( rect );
			top = rect.bottom + FORM_PADDING;
		}

	//
	// Reposition buttons
	CRect clientRect, posRect;
	m_submitButton.GetClientRect( &clientRect );

	posRect = clientRect;
	m_submitButton.MapWindowPoints( this, &posRect );

	m_submitButton.SetWindowPos
		( NULL,
			posRect.left,
			top + FORM_CONTROL_SPACING,
			clientRect.right,
			clientRect.bottom,
			SWP_NOZORDER );

	m_clearButton.GetClientRect( &clientRect );

	posRect = clientRect;
	m_clearButton.MapWindowPoints( this, &posRect );

	m_clearButton.SetWindowPos
		( NULL,
			posRect.left,
			top + FORM_CONTROL_SPACING,
			clientRect.right,
			clientRect.bottom,
			SWP_NOZORDER );

	//
	// All done
	CWnd::OnSize( nType, cx, cy );
}

/////////////////////////////////////////////////////////////////////////////
// Invoke the search associated with this form
void PG_FormWin::DoSearch()
{
	extern CPCPages32App theApp;
	theApp.SetMessageText( "" );

	if ( m_pRequest != NULL && m_pRequest->Finished() == TRUE )
		{
			delete m_pRequest;
			m_pRequest = NULL;
		}

	//
	// Set form search input 
	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_FormField *pField = field_list.GetNext( pos );
			pField->SetFieldValue();
		}

	//
	// Build a copy of the form search input arguments
	QFormInput *pTargetCopy;
	PtrList<QFormInput> contextCopy;

	//
	// Target field first 
	pTargetCopy = target_ffi->CopySelf();

	//
	// Context fields
	pos = context_ffi_list.GetHeadPosition();
	while ( pos != NULL )
    contextCopy.AddTail( context_ffi_list.GetNext( pos )->CopySelf() );

	//
	// Perform the search
	m_requestHandler.Reset();
	m_pRequest =
		new QFormSearch
			( theApp.DS(), theApp.DS()->GenRequestID(), &m_requestHandler, m_pForm,
				pTargetCopy, contextCopy );

	QError *pErr;
	if ( m_pRequest->Invoke( pErr ) == FALSE )
		{
			delete m_pRequest;
			if ( pErr != NULL )
				delete pErr;
		}
	else
		{
			//
			// Clear current search results
			theApp.EntryListDisplay().RemoveAllEntries();
			theApp.ShowDITDisplay();
			theApp.ShowEntryListDisplay();
		}
}

/////////////////////////////////////////////////////////////////////////////
// When the form window is shown (after being hidden) ensure that all
// controls fit properly. Also, set focus to the first edit control
// in the window.
void PG_FormWin::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CWnd::OnShowWindow( bShow, nStatus );

	//
	// Resize controls correctly when showing
	// the formw window
	if ( bShow == TRUE )
		{
			CRect rect;
			GetClientRect( &rect );

			UINT left = rect.left + FORM_PADDING;
			UINT right = rect.right - FORM_PADDING;
			UINT top = rect.top + FORM_PADDING;

			//
			// Repos and resize help strings
			if ( !m_pForm->TipString().IsEmpty() && m_pForm->TipString() != "" )
				{
					CClientDC dc( &m_staticTitle );
					CRect helpRect( left, top, right, top );
					dc.DrawText( m_pForm->TipString(), &helpRect,	DT_CALCRECT|DT_WORDBREAK );

					m_staticTitle.MoveWindow( &helpRect );
					top = helpRect.bottom + 8;
				}

			if ( !m_pForm->HelpString().IsEmpty() && m_pForm->HelpString() != "" )
				{
					CClientDC dc( &m_staticHelp );
					CRect helpRect( left, top, right, top );
					dc.DrawText( m_pForm->HelpString(), &helpRect,	DT_CALCRECT|DT_WORDBREAK );

					m_staticHelp.MoveWindow( &helpRect );
					top = helpRect.bottom + 8;
				}

			//
			// Now the input fields
			POSITION pos = field_list.GetHeadPosition();
			while ( pos != NULL )
				{
					PG_FormField *field = field_list.GetNext( pos );
					field->SizeControls( this, left, right, top );

					field->GetRect( rect );
					top = rect.bottom + FORM_PADDING;
				}

			if ( m_submitButton.GetSafeHwnd() != NULL )
				{
					//
					// Move the submit button to the right place below the form
					CRect submitRect;
					m_submitButton.GetClientRect( &submitRect );

					submitRect.left = FORM_CONTROL_SPACING;
					submitRect.right += submitRect.left;

					submitRect.top = top + FORM_CONTROL_SPACING;
					submitRect.bottom += submitRect.top;

					m_submitButton.MoveWindow( &submitRect, TRUE );
					m_submitButton.ShowWindow( SW_SHOW );

					//
					// Same with the clear button
					CRect clearRect;
					m_clearButton.GetClientRect( clearRect );

					clearRect.left = submitRect.right + FORM_CONTROL_SPACING;
					clearRect.right += clearRect.left;

					clearRect.top = top + FORM_CONTROL_SPACING;
					clearRect.bottom += clearRect.top;

					m_clearButton.MoveWindow( &clearRect, TRUE );
					m_submitButton.ShowWindow( SW_SHOW );
				}

			//
			// Set keyboard focus to the first edit control
			CWnd *pCtlWnd = GetDlgItem( FIRST_EDIT_CONTROL+1 );
			if ( pCtlWnd != NULL )
				pCtlWnd->SetFocus();
		}
}

/////////////////////////////////////////////////////////////////////////////
// Submit the search form.
void PG_FormWin::OnSubmit()
{
	DoSearch();
}

/////////////////////////////////////////////////////////////////////////////
// Clear the form.
void PG_FormWin::OnClear()
{
	POSITION pos = field_list.GetHeadPosition();
	while ( pos != NULL )
		{
			PG_FormField *field = field_list.GetNext( pos );
			field->Clear();
		}
}

