// QSearchClass.cpp: implementation of the QSearchClass class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "QSearchClass.h"
#include "Oidtable.h"
#include "QAttr.h"
#include "QDSManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QSearchClass::QSearchClass
  ( Oidtable *table, CString &name, PtrList<QOC> &oc_list )
{
	WIN32_GUFF();

  is_specific = FALSE;
	this->table = table;
  this->name = name;

  POSITION pos = oc_list.GetHeadPosition();
  while ( pos != NULL )
    this->oc_list.AddTail( oc_list.GetNext( pos ) );
}

/////////////////////////////////////////////////////////////////////////////////
// Used when class is specific, i.e. represents a named entry
/////////////////////////////////////////////////////////////////////////////////
QSearchClass::QSearchClass( QDSManager *ds, CString &name_arg, CString &ldap_uri )
{
	WIN32_GUFF();

	name = name_arg;
	table = ds->Table();

	LDAPURLDesc *ludp = NULL;
	char *uri_str = strdup( (LPCTSTR) ldap_uri );

	int url_ok = ldap_url_parse( uri_str, &ludp );
	if ( url_ok == LDAP_SUCCESS )
		{
			//
			// Get host details from URL (set authentication to none)
			CString host, user, password;
			host = ludp->lud_host;
			port = ludp->lud_port;
			user.Empty();
			password.Empty();

			ConnID conn_id = 
				ds->BindManager()->SpecifyConnection( host, ludp->lud_port, user, password );
			
			//
			// Set base DN
			dn.UnsetDN();
			dn.SetDN( ds, conn_id, CString( ludp->lud_dn ) );
		}

	if ( ludp != NULL )
		ldap_free_urldesc( ludp );

	free( uri_str );
	is_specific = TRUE;
}

QSearchClass::QSearchClass
  ( QDSManager *ds,
		CString &name_arg,
		CString &dn_str,
		CString &host,
		int port,
		CString &user,
		CString &password )
{
	WIN32_GUFF();

	name = name_arg;
	table = ds->Table();
	ConnID conn_id = ds->BindManager()->SpecifyConnection( host, port, user, password );
  this->dn.SetDN( ds, conn_id, dn_str );
	is_specific = TRUE;
}

QSearchClass::~QSearchClass()
{
}

/////////////////////////////////////////////////////////////////////////////////
// Add a class of entry that can be searched for.
void QSearchClass::AddSearchClass( QSearchClass *sc )
{
	WIN32_GUFF();
  srch_class_list.AddTail( sc );
}

/////////////////////////////////////////////////////////////////////////////////
// Get the list of search classes that can be searched for.
UINT QSearchClass::GetSearchClassList( PtrList<QSearchClass> &class_list )
{
	WIN32_GUFF();

  class_list.RemoveAll();
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    {
      class_list.AddTail( srch_class_list.GetNext( pos ) );
    }

  return class_list.GetCount();
}

/////////////////////////////////////////////////////////////////////////////////
// Generate a filter string that will match ONE OF the set of objectclasses
// defining this search class.
BOOL QSearchClass::GenerateFilterString( CString &oc_filt_str )
{
	WIN32_GUFF();

	if ( oc_list.GetCount() > 1 )
		oc_filt_str = "(|";
	else
		oc_filt_str = "";

	POSITION pos = oc_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QOC *oc = oc_list.GetNext( pos );
			oc_filt_str += ( "(objectclass=" + oc->Name() + ")" );
		}

	if ( oc_list.GetCount() > 1 )
		oc_filt_str += ")";

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////
// Check if the given object class defines an entry of this search class.
BOOL QSearchClass::HasObjectClass( QOC *oc )
{
	WIN32_GUFF();

  POSITION pos = oc_list.GetHeadPosition();
  while ( pos != NULL )
    {
      if ( oc == oc_list.GetNext( pos ) )
        return TRUE;
    }

	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////////
// Check if the given object class defines an entry of this search class.
BOOL QSearchClass::HasObjectClass( QAttr *oc_attr )
{
	WIN32_GUFF();

  //
  // Retrieve objectClass values
  PtrList<QValue> val_list;
  oc_attr->GetValues( val_list );

  //
  // Iterate list of object class values, comparing with
  // objectClass'es that define the search class (this->oc_list)
  POSITION pos = val_list.GetHeadPosition();
  while ( pos != NULL )
    {
      CString oc_str = ( val_list.GetNext( pos ) )->Value();
      QOC *curr_oc = table->str2oc( oc_str );

      if ( HasObjectClass( curr_oc ) == TRUE )
        return TRUE;
    }

	return FALSE;
}

//////////////////////////////////////////////////////////////////////////////////////
// Check if the given search class is searchable beneath objects of this search class.
BOOL QSearchClass::CanSearchFor( QSearchClass *check_sc )
{
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QSearchClass *sc = srch_class_list.GetNext( pos );
      if ( sc == check_sc )
        return TRUE;
    }

  return FALSE;
}

//////////////////////////////////////////////////////////////////////////////////////
// Return the base DN if the search class is specific to a named entry.
BOOL QSearchClass::IsSpecific( QDN &dn_ref )
{
  if ( is_specific )
    {
      dn_ref = dn;
      return TRUE;
    }

  return FALSE;
}
