// QEntry.cpp: implementation of the QEntry class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QEntry.h"
#include "QDSManager.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QEntry::QEntry( QDN &dn_arg, QAttr &oc_attr_arg, PtrList<QAttr> &attr_list_arg )
{
  WIN32_GUFF();

  // Set DN
  dn = dn_arg;

  // Set object class attribute
  oc_attr = oc_attr_arg.Copy();

  // Set remaining attrbiutes
  POSITION pos = attr_list_arg.GetHeadPosition();
  while ( pos != NULL )
    attr_list.AddTail( ( attr_list_arg.GetNext( pos ) )->Copy() );

  is_set = TRUE;
}

QEntry::QEntry( QDSManager *ds, ConnID conn_id, LDAP *ld, LDAPMessage *lm )
{
  WIN32_GUFF();

  is_set = FALSE;
	oc_attr = NULL;

  // Get the DN
  char *dn_str = ldap_get_dn( ld, lm );
  if ( dn_str == NULL )
    {
      // XXX Invalid LDAPMessage (not an LDAP_RES_SEARCH_ENTRY) passed???
      return;
    }
  else
    {
      // XXX Duff DN.
      if ( dn.SetDN( ds, conn_id, CString( dn_str ) ) == FALSE )
        return;
    }

  //
  // Convert attribute lists
  QAttrT *ocat = ds->Table()->str2at( CString( "objectclass" ) );
  BerElement *ber;
  for ( char *a = ldap_first_attribute( ld, lm, &ber );
        a != NULL;
        a = ldap_next_attribute( ld, lm, ber ) )
    {
      //
      // Create attribute structure
      QAttrT *at = ds->Table()->str2at( CString( a ) );
      if ( at == NULL )
        // XXX : Log failure to convert string to QAttrT (QEntry::QEntry)
        continue;
      
      QAttr *attr = new QAttr( at );

      //
      // Convert string values for each attribute.
			char **vals;
      if ( ( vals = ldap_get_values( ld, lm, a ) ) != NULL )
        {
          for ( int i = 0; vals[i] != NULL; i++ )
            attr->AddValue( vals[i], strlen( vals[i] ) + 1 );

          ldap_value_free( vals );
        }

      //
      // Add attribute to list or set object class
      if ( at == ocat )
        oc_attr = attr;
      else
        attr_list.AddTail( attr );
      
      ldap_memfree( a );
    }

  is_set = TRUE;
}

QEntry::~QEntry()
{
  WIN32_GUFF();

  POSITION pos = attr_list.GetHeadPosition();
  while ( pos != NULL )
    delete attr_list.GetNext( pos );

  if ( oc_attr != NULL )
    delete oc_attr;
}

//////////////////////////////////////////////////////////////////////
// Attribute retrieval
//////////////////////////////////////////////////////////////////////

POSITION QEntry::GetHeadPosition()
{
  WIN32_GUFF();

  return attr_list.GetHeadPosition();
}

QAttr *QEntry::GetNextAttr( POSITION &pos )
{
  WIN32_GUFF();

  return attr_list.GetNext( pos );
}

QAttr *QEntry::QueryGetAttr( QAttrT *pAT )
{
	WIN32_GUFF();

	POSITION pos = attr_list.GetHeadPosition();
	while ( pos != NULL )
		{
			QAttr *pAttr = attr_list.GetNext( pos );
			if ( pAttr->Type() == pAT )
				return pAttr;
		}

	return NULL;
}