// QLogFile.cpp: implementation of the QLogFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QLogFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QLogFile *QLogFile::c_pLogFile = NULL;
QLogFile::QLogFile( const char *pFileName )
:
CFile
	( pFileName,
		CFile::modeCreate|CFile::modeNoTruncate|CFile::modeWrite|CFile::shareDenyNone )
{
}

QLogFile::~QLogFile()
{
	ASSERT( c_pLogFile == this );
	c_pLogFile = NULL;
}

BOOL QLogFile::Open( const char *pFileName )
{
  WIN32_GUFF();
	c_pLogFile = new QLogFile( pFileName );
	c_pLogFile->SeekToEnd();
	return TRUE;
}

void QLogFile::Close()
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	c_pLogFile->CFile::Close();
	delete c_pLogFile;
}

void QLogFile::Log( const char *pStr, ... )
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	va_list argList;
	va_start( argList, pStr );

	char buf[4096];
	vsprintf( buf, pStr, argList );

	va_end( argList );

	CTime t = CTime::GetCurrentTime();
	CString timeStr =	t.Format( "%d/%m %H:%M:%S " );
	
	c_pLogFile->WriteString( "--------------------------------------------------------\n" );
	c_pLogFile->WriteString( timeStr );
	c_pLogFile->WriteString( buf );
	c_pLogFile->WriteString( "\n" );

	c_pLogFile->Flush();
}

void QLogFile::LogRaw( const char *pStr, ... )
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	va_list argList;
	va_start( argList, pStr );

	char buf[4096];
	vsprintf( buf, pStr, argList );

	va_end( argList );

	c_pLogFile->WriteString( buf );
	c_pLogFile->WriteString( "\n" );

	c_pLogFile->Flush();
}


void QLogFile::LogV(const char *pStr, va_list argList)
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	char buf[4096];
	vsprintf( buf, pStr, argList );

	CTime t = CTime::GetCurrentTime();
	CString timeStr =	t.Format( "%d/%m %H:%M:%S " );
	
	c_pLogFile->WriteString( timeStr );
	c_pLogFile->WriteString( buf );
	c_pLogFile->WriteString( "\n" );

	c_pLogFile->Flush();
}

void QLogFile::LogRawV(const char *pStr, va_list argList)
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	char buf[4096];
	vsprintf( buf, pStr, argList );

	c_pLogFile->WriteString( buf );
	c_pLogFile->WriteString( "\n" );

	c_pLogFile->Flush();
}

void QLogFile::WriteString( const char *pStr )
{
  WIN32_GUFF();

	if ( c_pLogFile == NULL )
		return;

	unsigned int nLen = strlen( pStr );
	c_pLogFile->Write( reinterpret_cast<const void *>( pStr ), nLen );
}
